@@ -1,6 +1,10 @@
 from functools import wraps
 from typing import Dict, List, Sequence, Union
 
+# This is a value that will never match any operator. It is useful when evaluating multiple
+# rule sets and want to ignore rule sets targeting a specific field
+NEVER_MATCH = object()
+
 
 class JsonOperationError(Exception):
     pass
@@ -23,7 +27,22 @@ def get_json_schema() -> Dict:
                 "minItems": 3,
                 "maxItems": 3,
                 "items": [
-                    {"enum": ["=", "==", "!=", ">", ">=", "<", "<=", "in", "nin"]},
+                    {
+                        "enum": [
+                            "=",
+                            "==",
+                            "!=",
+                            ">",
+                            ">=",
+                            "<",
+                            "<=",
+                            "in",
+                            "nin",
+                            "!in",
+                            "btw",
+                            "&",
+                        ]
+                    },
                     {
                         "$def": "#/definitions/operationOrLiteral",
                     },
@@ -76,11 +95,11 @@ def _use_type_from_val(operator):
 
 
 def _get_type_from_operator(operator, index):
-    if operator in {">", "<", ">=", "<="}:
+    if operator in {">", "<", ">=", "<=", "btw"}:
         return "number"
     elif operator in {"=", "==", "!="}:
         return ["number", "string", "boolean"]
-    elif operator in {"in", "nin"}:
+    elif operator in {"in", "nin", "!in"}:
         if index == 0:
             return ["string", "number", "boolean"]
         elif index == 1:
@@ -99,7 +118,7 @@ def _in(needle, stack):
     return needle in stack
 
 
-def _nin(needle, stack):
+def _not_in(needle, stack):
     # "not in" operator
     return needle not in stack
 
@@ -108,23 +127,25 @@ def _is_number(a):
     return isinstance(a, (int, float)) and not isinstance(a, bool)
 
 
-def _same_type(a, b):
+def _same_type(*args):
     # Different number types aren't the same type, so bucket all numbers
-    if _is_number(a) and _is_number(b):
+    if all(_is_number(arg) for arg in args):
         return True
 
-    return type(a) == type(b)
+    # Check if all types are the same
+    return len({type(arg) for arg in args}) == 1
 
 
 def _raise_type_error(operator_str):
     def dec(fn):
         @wraps(fn)
-        def inner(a, b):
-            if not _same_type(a, b):
+        def inner(*args):
+            if not _same_type(*args):
+                error_type_msg = " and ".join(type(arg).__name__ for arg in args)
                 raise TypeError(
-                    f"'{operator_str}' not supported between instances of '{type(a).__name__}' and '{type(b).__name__}'"
+                    f"'{operator_str}' not supported between instances of {error_type_msg}"
                 )
-            return fn(a, b)
+            return fn(*args)
 
         return inner
 
@@ -156,6 +177,24 @@ def _less_or_equal(a, b):
     return a <= b
 
 
+def _between(val, range):
+    if not isinstance(range, list):
+        raise TypeError("btw list is not a list")
+    if len(range) != 2 or not _is_number(range[0]) or not _is_number(range[1]):
+        raise TypeError("btw list must have to number type items")
+    if not _is_number(val):
+        raise TypeError(f"btw cannot be used with type {type(val)}")
+
+    return range[0] <= val <= range[1]
+
+
+def _intersection(a, b):
+    if not isinstance(a, list) or not isinstance(b, list):
+        raise TypeError("intersection operator must be used with 2 lists")
+
+    return bool(set(a) & set(b))
+
+
 @_raise_type_error("!=")
 def _not_equal(a, b):
     if not _same_type(a, b):
@@ -187,7 +226,10 @@ _operators = {
     "null": _null_,
     "!null": _not_null,
     "in": _in,
-    "nin": _nin,
+    "nin": _not_in,
+    "!in": _not_in,
+    "btw": _between,
+    "&": _intersection,
 }
 
 _nesting_operators = {"and", "or"}
@@ -207,6 +249,13 @@ def _get_key(context, key, default=None):
         return context
 
 
+def _execute_operation(operation: str, values):
+    if NEVER_MATCH in values:
+        return False
+
+    return _operators[operation](*values)
+
+
 def get_keys(json_operation: List) -> List[Dict]:
     operator, *unparsed = json_operation
 
@@ -288,7 +337,7 @@ def _execute_base(json_operation: List, context, handler, prefix=""):
         raise JsonOperationError(f"Invalid operator: {operator}. {json_operation}")
 
     try:
-        return handler(_operators[operator](*values), prefix)
+        return handler(_execute_operation(operator, values), prefix)
     except TypeError as e:
         raise JsonOperationError(f"{e}. {json_operation}")
 
